@@ -5,6 +5,7 @@ require "fileutils"
 
 $git_mutex     = Mutex.new
 $message_mutex = Mutex.new
+$action_width  = 12
 
 def puts(message)
   $message_mutex.synchronize {
@@ -13,18 +14,19 @@ def puts(message)
   }
 end
 
-def puts_action(action, target=nil)
+def puts_action(action, options = {})
   msg  = sprintf "%*s", $action_width, action.gsub(/ +/, '_').upcase
-  msg += " #{target}" if target && !target.empty?
+  msg += " #{options[:target]}"       if !options[:target].blank?
+  msg  = "#{options[:prefix]} #{msg}" if !options[:prefix].blank?
   puts msg
 end
 
-def puts_qaction(action, target=nil)
-  puts_action(action, target) unless $verbose
+def puts_qaction(action, options = {})
+  puts_action(action, options) unless $verbose
 end
 
-def puts_vaction(action, target=nil)
-  puts_action(action, target) if $verbose
+def puts_vaction(action, options = {})
+  puts_action(action, options) if $verbose
 end
 
 def error(message)
@@ -80,18 +82,36 @@ ensure
 end
 
 def runq(action, target, cmdline, options = {})
-  puts_qaction action, target
-  run cmdline, options.clone.merge(:dont_echo => !$verbose)
+  start  = Time.now
+  runner = lambda { run_wrapper cmdline, options.clone.merge(:dont_echo => !$verbose) }
+
+  puts_action action, :target => target, :prefix => $run_show_start_stop ? "[start          ]" : nil
+
+  code     = options[:mutex] ? options[:mutex].synchronize(&runner) : runner.call
+  duration = Time.now - start
+
+  if $run_show_start_stop
+    puts_action action, :target => target, :prefix => sprintf("[%s  %02d:%02d.%03d]", code == 0 ? "done" : "fail", (duration / 60).to_i, duration.to_i % 60, (duration * 1000).to_i % 1000)
+  end
+
+  if (code != 0) && !options[:allow_failure].to_bool
+    if code == 127
+      puts "Error: command not found: #{cmdline}"
+    elsif code == -1
+      puts "Error: could not create child process for: #{cmdline}"
+    end
+
+    exit code
+  end
 end
 
 def runq_git(msg, cmdline, options = {})
-  puts_qaction "git #{cmdline.split(/\s+/)[0]}", "#{msg}"
-  $git_mutex.synchronize { run "git #{cmdline}", options.clone.merge(:dont_echo => !$verbose) }
+  runq "git #{cmdline.split(/\s+/)[0]}", "#{msg}", "git #{cmdline}", :mutex => $git_mutex
 end
 
 def ensure_dir dir
-  File.unlink(dir) if  FileTest.exist?(dir) && !FileTest.directory?(dir)
-  Dir.mkdir(dir)   if !FileTest.exist?(dir)
+  File.unlink(dir) if FileTest.exist?(dir) && !FileTest.directory?(dir)
+  FileUtils.mkdir_p(dir)
 end
 
 def create_dependency_dirs
@@ -169,7 +189,7 @@ end
 
 def remove_files_by_patterns patterns
   patterns.collect { |pattern| FileList[pattern].to_a }.flatten.uniq.select { |file_name| File.exists? file_name }.each do |file_name|
-    puts_vaction "rm", file_name
+    puts_vaction "rm", :target => file_name
     File.unlink file_name
   end
 end
@@ -178,7 +198,29 @@ def read_files *file_names
   Hash[ *file_names.flatten.collect { |file_name| [ file_name, IO.readlines(file_name).collect { |line| line.force_encoding "UTF-8" } ] }.flatten(1) ]
 end
 
+def list_targets? *targets
+  spec = ENV['RAKE_LIST_TARGETS'].to_s
+  targets.any? { |target| %r{\b#{target}\b}.match spec }
+end
+
 class Rake::Task
+  def mo_all_prerequisites
+    todo   = [name]
+    result = []
+
+    while !todo.empty?
+      current = todo.shift
+      prereqs = Rake::Task[current].prerequisites
+
+      next if prereqs.empty?
+
+      result << [current, prereqs]
+      todo   += prereqs
+    end
+
+    result.uniq
+  end
+
   def investigate
     result = "------------------------------\n"
     result << "Investigating #{name}\n"
