@@ -1,5 +1,4 @@
-#ifndef MTX_MKVTOOLNIX_GUI_UTIL_CONTAINER_H
-#define MTX_MKVTOOLNIX_GUI_UTIL_CONTAINER_H
+#pragma once
 
 #include "common/common_pch.h"
 
@@ -12,7 +11,7 @@ template<typename Tstored, typename Tcontainer>
 int
 findPtr(Tstored *needle,
         Tcontainer const &haystack) {
-  auto itr = brng::find_if(haystack, [&](std::shared_ptr<Tstored> const &cmp) { return cmp.get() == needle; });
+  auto itr = brng::find_if(haystack, [needle](auto const &cmp) { return cmp.get() == needle; });
   return haystack.end() == itr ? -1 : std::distance(haystack.begin(), itr);
 }
 
@@ -20,5 +19,3 @@ std::vector<std::string> toStdStringVector(QStringList const &strings, int offse
 QStringList toStringList(std::vector<std::string> const &stdStrings, int offset = 0);
 
 }}}
-
-#endif  // MTX_MKVTOOLNIX_GUI_UTIL_CONTAINER_H
